/**
 * @file list.c
 * @brief Implementacao basica de uma lista circular.
 */
#include <stdlib.h>

#include "list.h"

/**
 * @brief Funcao para insercao.
 *
 * @param 'head' identifica a cabeca da lista onde inserir.
 * @param 'value' identifica o valor a inserir.
 */
void
list_insert(link head, void * value)
{
  link node = (link) malloc(sizeof(struct node));
  node->value = value;
  node->next = head;
  node->prev = head->prev;
  head->prev->next = node;
  head->prev = node;
}

/**
 * @brief Funcao para mover um no' entre listas.
 *
 * ATENCAO: Nao verifica se o no' ja' faz parte da lista!
 *
 * @param 'head' identifica a cabeca da lista onde mover.
 * @param 'node' identifica o no' a mover.
 */
void
list_move(link head, link node)
{
  /* Remove */
  node->prev->next = node->next;
  node->next->prev = node->prev;

  /* Add */
  node->next = head;
  node->prev = head->prev;
  head->prev->next = node;
  head->prev = node;
}

/**
 * @brief Funcao para remover um no' da lista.
 *
 * Neste caso nao e' necessaria a cabeca da lista uma vez que estamos
 * a utilizar listas duplamente ligadas.
 *
 * @param 'node' identifica o no' a mover.
 */
void
list_delete(link node)
{
  node->prev->next = node->next;
  node->next->prev = node->prev;
  free(node);
}

/**
 * @brief Funcao para libertar a memoria associada a uma lista.
 *
 * @param 'head' identifica a cabeca da lista a libertar.
 * @param 'freevalue' identifica a funcao que liberta a memoria
 *        associada aos valores (pode ser NULL).
 */
void
list_free(link head, void (*freevalue)(void *))
{
  link l = head->next, aux;

  while (l != head) {
    if (freevalue)
      (*freevalue)(l->value);
    aux = l;
    l = l->next;
    free(aux);
  }
}

